@@ -25,51 +25,21 @@
 #ifndef INCLUDED_MCDB_H
 #define INCLUDED_MCDB_H
 
-#ifdef _AIX  /*mmap constants and basic networking on AIX require non-standard*/
-#ifndef _ALL_SOURCE
-#define _ALL_SOURCE
-#endif
-#endif
-#ifdef __hpux /*HP-UX strict conformance with C99 and _XOPEN_SOURCE 600*/
-#ifndef _XOPEN_SOURCE
-#define _XOPEN_SOURCE 600
-#endif
-#endif
+#include "plasma/plasma_feature.h"
+#include "plasma/plasma_attr.h"
+#include "plasma/plasma_stdtypes.h" /* bool, size_t, uint32_t, uintptr_t */
+PLASMA_ATTR_Pragma_once
 
-#include <stdint.h>   /* uint32_t, uintptr_t */
-#include <unistd.h>   /* size_t   */
-#include <sys/time.h> /* time_t   */
-
-#ifndef __cplusplus
-#if __STDC_VERSION__ >= 199901L /* C99 */
-#if defined(bool) && !defined(__bool_true_false_are_defined)
-#undef bool
-#endif
-#include <stdbool.h>  /* bool */
-#else
-/* http://pubs.opengroup.org/onlinepubs/007904875/basedefs/stdbool.h.html */
-/*(intentionally conflict w/ existing #define bool if different and no _Bool)*/
-#define bool char
-#define true  1
-#define false 0
-/*#define __bool_true_false_are_defined 1 *//* not defining since not _Bool */
-#endif
-#endif
-
-#if defined(__cplusplus) || __STDC_VERSION__ < 199901L /* C99 */
-#ifdef __GNUC__
-#define restrict __restrict
-#else
-#define restrict
-#endif
-#endif
+#include <sys/time.h>               /* time_t */
 
+#ifdef PLASMA_FEATURE_POSIX
+#include <unistd.h>                 /* _POSIX_* features */
+/* http://pubs.opengroup.org/onlinepubs/7908799/xsh/feature.html */
 #if !defined(_POSIX_MAPPED_FILES) || !(_POSIX_MAPPED_FILES-0) \
  || !defined(_POSIX_SYNCHRONIZED_IO) || !(_POSIX_SYNCHRONIZED_IO-0)
 #error "mcdb requires mmap() and msync() support"
 #endif
-
-#include "code_attributes.h"
+#endif
 
 #ifdef __cplusplus
 extern "C" {
@@ -79,16 +49,24 @@ struct mcdb_mmap {
   unsigned char *ptr;         /* mmap pointer */
   uint32_t b;                 /* hash table stride bits: (data < 4GB) ? 3 : 4 */
   uint32_t n;                 /* num records in mcdb */
+  uint32_t hash_init;         /* hash init value */
+  uint32_t hash_pad;          /* (padding)*/
+  uint32_t (*hash_fn)(uint32_t, const void * restrict, size_t); /* hash func */
   uintptr_t size;             /* mmap size */
   time_t mtime;               /* mmap file mtime */
-  struct mcdb_mmap * volatile next;    /* updated (new) mcdb_mmap */
-  void * (*fn_malloc)(size_t);         /* fn ptr to malloc() */
-  void (*fn_free)(void *);             /* fn ptr to free() */
-  volatile uint32_t refcnt;            /* registered access reference count */
-  int dfd;                    /* fd open to dir in which mmap file resides */
+  struct mcdb_mmap *next;     /* updated (new) mcdb_mmap */
+  void * (*fn_malloc)(size_t);/* fn ptr to malloc() */
+  void (*fn_free)(void *);    /* fn ptr to free() */
   char *fname;                /* basename of mmap file, relative to dir fd */
-  char fnamebuf[64];          /* buffer in which to store short fname */
+  char fnamebuf[112];         /* buffer in which to store short fname */
+  int allocated;              /* flag if struct allocated in mcdb_mmap_create */
+  int dfd;                    /* fd open to dir in which mmap file resides */
+  uint32_t refcnt;            /* registered access reference count */
 };
+/* aside: char fnamebuf[] sized to separate 'next' and 'refcnt' by 128 bytes
+ * (L2 cache lines on modern hardware are 64-bytes and 128-bytes)
+ * (32-bit pointers are 4-byte; 64-bit pointers are 8-byte)
+ * (separate cache lines for high-frequency read-only data and modified data) */
 
 struct mcdb {
   struct mcdb_mmap *map;
@@ -164,7 +142,8 @@ extern void
 mcdb_iter_init(struct mcdb_iter * restrict, struct mcdb * restrict)
   __attribute_nonnull__  __attribute_nothrow__;
 
-extern struct mcdb_mmap *  __attribute_malloc__
+__attribute_malloc__
+extern struct mcdb_mmap *
 mcdb_mmap_create(struct mcdb_mmap * restrict,
                  const char *,const char *,void * (*)(size_t),void (*)(void *))
   __attribute_nonnull_x__((3,4,5))  __attribute_warn_unused_result__;
@@ -208,12 +187,10 @@ mcdb_mmap_refresh_check(const struct mcdb_mmap * restrict)
 enum mcdb_flags {
   MCDB_REGISTER_USE_DECR = 0,
   MCDB_REGISTER_USE_INCR = 1,
-  MCDB_REGISTER_MUNMAP_SKIP = 2,
-  MCDB_REGISTER_MUTEX_LOCK_HOLD = 4,
-  MCDB_REGISTER_MUTEX_UNLOCK_HOLD = 8
+  MCDB_REGISTER_ALREADY_LOCKED = 2
 };
 
-extern bool
+extern struct mcdb_mmap *
 mcdb_mmap_thread_registration(struct mcdb_mmap ** restrict, int)
   __attribute_nonnull__;
 extern bool
@@ -229,7 +206,7 @@ mcdb_mmap_reopen_threadsafe(struct mcdb_mmap ** restrict)
   mcdb_mmap_refresh_threadsafe(&(mcdb)->map)
 #define mcdb_thread_refresh_self(mcdb) \
   (__builtin_expect((mcdb)->map->next == NULL, true) \
-   || __builtin_expect(mcdb_thread_register(mcdb), true))
+   || __builtin_expect(mcdb_thread_register(mcdb) != NULL, true))
 
 
 #define MCDB_SLOT_BITS 8                  /* 2^8 = 256 */
@@ -248,7 +225,7 @@ mcdb_mmap_reopen_threadsafe(struct mcdb_mmap ** restrict)
  * (optimization)
  * The aliases below are not a complete set of mcdb symbols,
  * but instead are the most common used in libnss_mcdb.so.2 */
-#if __GNUC_PREREQ(4,0) || __has_attribute(alias)
+#ifdef PLASMA_ATTR_ALIAS
 HIDDEN extern __typeof (mcdb_findtagstart)
                         mcdb_findtagstart_h;
 HIDDEN extern __typeof (mcdb_findtagnext)
