@@ -25,7 +25,9 @@
 #ifndef INCLUDED_MCDB_ERROR_H
 #define INCLUDED_MCDB_ERROR_H
 
-#include "code_attributes.h"
+#include "plasma/plasma_feature.h"
+#include "plasma/plasma_attr.h"
+PLASMA_ATTR_Pragma_once
 
 #ifdef __cplusplus
 extern "C" {
