@@ -33,47 +33,40 @@
 #ifndef _BSD_SOURCE
 #define _BSD_SOURCE
 #endif
+/* _DARWIN_C_SOURCE needed for mmap MAP_ANON on MacOSX */
+#define PLASMA_FEATURE_ENABLE_BSD_SOURCE_TO_DARWIN_C_SOURCE
 /* large file support needed for mmap() offset,ftruncate() on mcdb > 2 GB */
-#if defined(_AIX)
-#ifndef _LARGE_FILES
-#define _LARGE_FILES
-#endif
-#else /*#elif defined(__linux__) || defined(__sun) || defined(__hpux)*/
-#ifndef _FILE_OFFSET_BITS
-#define _FILE_OFFSET_BITS 64
-#endif
-#ifndef _LARGEFILE_SOURCE
-#define _LARGEFILE_SOURCE 1
-#endif
-#ifndef _LARGEFILE64_SOURCE
-#define _LARGEFILE64_SOURCE 1
-#endif
-#endif
-
-#ifdef __linux__
-#include <features.h>
-#if defined(__GLIBC__) && !defined(__clang__)
-#define __USE_STRING_INLINES
-#endif
-#endif
+#define PLASMA_FEATURE_ENABLE_LARGEFILE
+/* enable extra glibc string inlines
+ * (tested and improves perf for mcdb_make.c) */
+#define PLASMA_FEATURE_ENABLE_GLIBC_STRING_INLINES
 
 #include "mcdb_make.h"
 #include "mcdb.h"
 #include "nointr.h"
 #include "uint32.h"
-#include "code_attributes.h"
+#include "plasma/plasma_stdtypes.h"
 
-#include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/mman.h>
 #include <unistd.h>
 #include <errno.h>
 #include <fcntl.h>   /* posix_fallocate() */
 #include <string.h>  /* memcpy() */
-#include <stdbool.h> /* bool true false */
-#include <stdint.h>  /* uint32_t uintptr_t */
 #include <limits.h>  /* UINT_MAX, INT_MAX */
 
+#ifdef _AIX
+#ifndef MAP_ANONYMOUS
+#define MAP_ANONYMOUS 0x10
+#endif
+#endif
+
+#if defined(__APPLE__) && defined(__MACH__)
+#ifndef MAP_ANONYMOUS
+#define MAP_ANONYMOUS MAP_ANON
+#endif
+#endif
+
 /*(posix_madvise, defines not provided in Solaris 10, even w/ __EXTENSIONS__)*/
 #if (defined(__sun) || defined(__hpux)) && !defined(POSIX_MADV_NORMAL)
 extern int madvise(caddr_t, size_t, int);
@@ -96,7 +89,8 @@ struct mcdb_hplist {
 
 /* routine marked to indicate unlikely branch;
  * __attribute_cold__ can be used instead of __builtin_expect() */
-static int  __attribute_noinline__  __attribute_cold__
+__attribute_noinline__  __attribute_cold__
+static int
 mcdb_make_err(struct mcdb_make * const restrict m, int errnum)
 {
     if (m != NULL) mcdb_make_destroy(m);
@@ -104,7 +98,8 @@ mcdb_make_err(struct mcdb_make * const restrict m, int errnum)
     return -1;
 }
 
-static bool  __attribute_noinline__
+__attribute_noinline__
+static bool
 mcdb_hplist_alloc(struct mcdb_make * const restrict m)
   __attribute_nonnull__  __attribute_warn_unused_result__;
 static bool
@@ -151,7 +146,8 @@ mcdb_hplist_alloc(struct mcdb_make * const restrict m)
 #define FSTYPSZ 16
 #endif
 #include <sys/statvfs.h>
-static int  __attribute_noinline__
+__attribute_noinline__
+static int
 mcdb_make_fallocate(const int fd, off_t offset, off_t len)
   __attribute_warn_unused_result__;
 static int
@@ -249,11 +245,13 @@ mcdb_mmap_commit(struct mcdb_make * const restrict m,
      * OS crashes, then the updated mcdb can be corrupted. */
 }
 
-static bool  __attribute_noinline__
+__attribute_noinline__
+static bool
 mcdb_mmap_upsize(struct mcdb_make * const restrict m, const size_t sz,
                  const bool sequential)
   __attribute_nonnull__  __attribute_warn_unused_result__;
-static bool  __attribute_noinline__
+__attribute_noinline__
+static bool
 mcdb_mmap_upsize(struct mcdb_make * const restrict m, const size_t sz,
                  const bool sequential)
 {
@@ -276,9 +274,12 @@ mcdb_mmap_upsize(struct mcdb_make * const restrict m, const size_t sz,
         msz = (UINT_MAX & m->pgalign) - offset;
   #endif
 
-    /* increase file size by at least msz (prefer multiple of disk block size)*/
+    /* increase file size by at least msz (prefer multiple of disk block size)
+     * (reduce to MCDB_MMAP_SZ for 1st (and maybe 2nd) mmap for small mcdb) */
     if (m->fd != -1 && m->fsz < offset + msz) {
-        m->fsz = (offset + msz + (MCDB_BLOCK_SZ-1)) & ~(MCDB_BLOCK_SZ-1);
+        m->fsz = (m->offset != 0)
+          ? ((offset + msz + (MCDB_BLOCK_SZ-1)) & ~(size_t)(MCDB_BLOCK_SZ-1))
+          : ((offset + msz + (MCDB_MMAP_SZ-1))  & ~(size_t)(MCDB_MMAP_SZ-1));
       #if defined(__GLIBC__)/* glibc emulates if not natively supported by fs */
         if ((errno = posix_fallocate(m->fd, (off_t)m->osz,
                                      (off_t)(m->fsz-m->osz))) == 0)
@@ -340,7 +341,7 @@ mcdb_make_addbegin(struct mcdb_make * const restrict m,
     if (m->map == MAP_FAILED && m->fd != -1)  return mcdb_make_err(NULL,EPERM);
     if (m->hp.l== ~0 && !mcdb_hplist_alloc(m))return mcdb_make_err(NULL,errno);
     m->hp.p = pos;
-    m->hp.h = UINT32_HASH_DJB_INIT;
+    m->hp.h = m->hash_init;
     if (keylen>INT_MAX-8 || datalen>INT_MAX-8)return mcdb_make_err(NULL,EINVAL);
     m->hp.l = (uint32_t)keylen;
   #if !defined(_LP64) && !defined(__LP64__)  /* (no 4 GB limit in 64-bit) */
@@ -371,7 +372,9 @@ mcdb_make_addbuf_key(struct mcdb_make * const restrict m,
 {
     /* len validated in mcdb_make_addbegin(); passing any other len is wrong,
      * unless the len is shorter from partial contents of buf. */
-    m->hp.h = uint32_hash_djb(m->hp.h, buf, len);
+    m->hp.h = (m->hash_fn == uint32_hash_djb)
+      ? uint32_hash_djb(m->hp.h, buf, len)
+      : m->hash_fn(m->hp.h, buf, len);
     mcdb_make_addbuf_data(m, buf, len);
 }
 
@@ -416,6 +419,8 @@ mcdb_make_start(struct mcdb_make * const restrict m, const int fd,
     m->map       = MAP_FAILED;
     m->pos       = MCDB_HEADER_SZ;
     m->offset    = 0;
+    m->hash_init = UINT32_HASH_DJB_INIT;
+    m->hash_fn   = uint32_hash_djb;
     m->fsz       = 0;
     m->osz       = 0;
     m->msz       = 0;
@@ -562,7 +567,8 @@ mcdb_make_finish(struct mcdb_make * const restrict m)
  * m->fd is not closed here since mcdb_make_start() takes open file descriptor
  * (caller should cleanup m->fd)
  */
-int __attribute_noinline__
+__attribute_noinline__
+int
 mcdb_make_destroy(struct mcdb_make * const restrict m)
 {
     int rc = 0;
@@ -602,20 +608,20 @@ mcdb_make_destroy(struct mcdb_make * const restrict m)
  * (Reference: "How to Write Shared Libraries", by Ulrich Drepper)
  * (optimization)
  * The aliases below are not a complete set of mcdb_make symbols */
-#if __GNUC_PREREQ(4,0) || __has_attribute(alias)
+#ifdef PLASMA_ATTR_ALIAS
 HIDDEN extern __typeof (mcdb_make_add)
                         mcdb_make_add_h
-  __attribute__((alias ("mcdb_make_add")));
+  __attribute_alias__ ("mcdb_make_add");
 HIDDEN extern __typeof (mcdb_make_addbegin)
                         mcdb_make_addbegin_h
-  __attribute__((alias ("mcdb_make_addbegin")));
+  __attribute_alias__ ("mcdb_make_addbegin");
 HIDDEN extern __typeof (mcdb_make_addbuf_data)
                         mcdb_make_addbuf_data_h
-  __attribute__((alias ("mcdb_make_addbuf_data")));
+  __attribute_alias__ ("mcdb_make_addbuf_data");
 HIDDEN extern __typeof (mcdb_make_addbuf_key)
                         mcdb_make_addbuf_key_h
-  __attribute__((alias ("mcdb_make_addbuf_key")));
+  __attribute_alias__ ("mcdb_make_addbuf_key");
 HIDDEN extern __typeof (mcdb_make_addend)
                         mcdb_make_addend_h
-  __attribute__((alias ("mcdb_make_addend")));
+  __attribute_alias__ ("mcdb_make_addend");
 #endif
