@@ -29,21 +29,7 @@
 #define _XOPEN_SOURCE 600
 #endif
 /* large file support needed for open() input file > 2 GB */
-#if defined(_AIX)
-#ifndef _LARGE_FILES
-#define _LARGE_FILES
-#endif
-#else /*#elif defined(__linux__) || defined(__sun) || defined(__hpux)*/
-#ifndef _FILE_OFFSET_BITS
-#define _FILE_OFFSET_BITS 64
-#endif
-#ifndef _LARGEFILE_SOURCE
-#define _LARGEFILE_SOURCE 1
-#endif
-#ifndef _LARGEFILE64_SOURCE
-#define _LARGEFILE64_SOURCE 1
-#endif
-#endif
+#define PLASMA_FEATURE_ENABLE_LARGEFILE
 
 #include "mcdb.h"
 #include "mcdb_makefmt.h"
@@ -51,18 +37,16 @@
 #include "mcdb_error.h"
 #include "nointr.h"
 #include "uint32.h"
+#include "plasma/plasma_stdtypes.h"
 
 #include <sys/mman.h>
-#include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>   /* open(), O_RDONLY */
-#include <stdbool.h> /* bool */
-#include <stdio.h>   /* printf(), snprintf(), IOV_MAX */
+#include <stdio.h>   /* printf(), IOV_MAX */
 #include <stdlib.h>  /* malloc(), free(), EXIT_SUCCESS */
 #include <string.h>  /* strlen() */
 #include <unistd.h>  /* STDIN_FILENO, STDOUT_FILENO _SC_PAGESIZE */
 #include <sys/uio.h> /* writev() */
-#include <libgen.h>  /* basename() */
 #include <limits.h>  /* SSIZE_MAX */
 
 /*(posix_madvise, defines not provided in Solaris 10, even w/ __EXTENSIONS__)*/
@@ -174,7 +158,7 @@ mcdbctl_dump(struct mcdb * const restrict m)
         ++iovcnt;
 
         iov[iovcnt].iov_base = buf+buflen;
-        buflen+=iov[iovcnt].iov_len=(uint32_t)snprintf(buf+buflen,11,"%u",klen);
+        buflen += iov[iovcnt].iov_len = uint32_to_ascii_base10(klen,buf+buflen);
         ++iovcnt;
 
         iov[iovcnt].iov_base = ",";
@@ -182,7 +166,7 @@ mcdbctl_dump(struct mcdb * const restrict m)
         ++iovcnt;
 
         iov[iovcnt].iov_base = buf+buflen;
-        buflen+=iov[iovcnt].iov_len=(uint32_t)snprintf(buf+buflen,11,"%u",dlen);
+        buflen += iov[iovcnt].iov_len = uint32_to_ascii_base10(dlen,buf+buflen);
         ++iovcnt;
 
         iov[iovcnt].iov_base = ":";
@@ -268,7 +252,6 @@ mcdbctl_stats(struct mcdb * const restrict m)
     for (rv = 0; rv < 10; ++rv)
         printf("d%d      %lu\n", rv, numd[rv]);
     printf(">9      %lu\n", numd[10]);
-    fflush(stdout);
     return EXIT_SUCCESS;
 }
 
@@ -553,10 +536,10 @@ static const char * const restrict mcdb_usage =
    "         mcdbctl uniq  <fname.mcdb> [\"first\"|\"last\"]\n"
    "         mcdbctl dump  <fname.mcdb>\n"
    "         mcdbctl stats <fname.mcdb>\n"
-   "         mcdbctl get   <fname.mcdb> <key> [seq]\n";
+   "         mcdbctl get   <fname.mcdb> <key> [seq|\"all\"]\n";
 
 /*
- * mcdbctl get   <mcdb> <key> [seq]
+ * mcdbctl get   <mcdb> <key> [seq|"all"]
  * mcdbctl dump  <mcdb>
  * mcdbctl stats <mcdb>
  * mcdbctl make  <mcdb> <input-file>
@@ -578,5 +561,5 @@ main(int argc, char ** const restrict argv)
 
     return rv == EXIT_SUCCESS
       ? EXIT_SUCCESS
-      : mcdb_error(rv, basename(argv[0]), mcdb_usage);
+      : mcdb_error(rv, "mcdbctl", mcdb_usage);
 }
